import { PageMeta, View, Input, Icon, ScrollView } from "@tarojs/components";
import { ConfigProvider, Empty, Flex, Navbar, Tabs, Tag } from "@taroify/core";
import { useEffect, useState } from "react";
import "./index.css";
import Taro, { useDidShow, useLoad } from "@tarojs/taro";

import useGlobalStore from "@/store/useUserStore";
import services from "@/services";

const List = () => {
  const [inputStr, setInputStr] = useState("");
  const [activeTab, setActiveTab] = useState(0);
  const [paginationInfo, setPaginationInfo] = useState<any>();
  const [backlogDataSource, setBacklogDataSource] = useState<any>([]);
  const { updateActionObj } = useGlobalStore();
  const [historyPaginationInfo, setHistoryPaginationInfo] = useState<any>();
  const [historyDataSource, setHistoryDataSource] = useState<any[]>([]);
  const [historyRefresherTriggered, setHistoryRefresherTriggered] =
    useState(false);
  const [refresherTriggered, setRefresherTriggered] = useState(false);
  useDidShow(() => {
    getData(1);
    getHistory(1);

    Taro.hideHomeButton({
      complete(res) {
        console.log(res);
      },
    });
  });

  const getData = (pageNum) => {
    const params = {
      name: inputStr,
      page: pageNum,
      count: 10,
    };
    services.login
      .getBacklog(params)
      .then((res: any) => {
        if (res.code === 200) {
          if (pageNum === 1) {
            setBacklogDataSource([...res.data?.items]);
          } else {
            setBacklogDataSource([...backlogDataSource, ...res.data?.items]);
          }
          setPaginationInfo(res.data);
        }
      })
      .finally(() => {
        setRefresherTriggered(false);
      });
  };

  const getHistory = (pageNum) => {
    const params = {
      name: inputStr,
      page: pageNum,
      count: 10,
    };
    services.login
      .getBacklogHistory(params)
      .then((res: any) => {
        if (res.code === 200) {
          if (pageNum === 1) {
            setHistoryDataSource([...res.data?.items]);
          } else {
            setHistoryDataSource([...historyDataSource, ...res.data?.items]);
          }

          setHistoryPaginationInfo(res.data);
        }
      })
      .catch((e) => {
        console.log(e);
      })
      .finally(() => {
        setHistoryRefresherTriggered(false);
      });
  };

  useEffect(() => {
    getData(1);
    getHistory(1);
  }, [inputStr]);

  const onScrollToLower = () => {
    if (paginationInfo.hasNext) {
      getData(Number(paginationInfo.page + 1));
    }
  };

  const onScrollToLowerInHistory = () => {
    if (historyPaginationInfo.hasNext) {
      getHistory(Number(historyPaginationInfo.page) + 1);
    }
  };

  const onClick = (params) => {
    updateActionObj(params);
    Taro.navigateTo({
      url: "/pages/detail/index?activeTab=" + activeTab,
    });
  };

  return (
    <View className="container">
      <View className="content">
        <View className="search-box">
          <View className="search-wrapper">
            <Icon type="search" size={14}></Icon>
            <Input
              placeholder="搜索申请单位、事项名称"
              className="input"
              onInput={(e) => setInputStr(e.detail.value)}
            />
          </View>
        </View>
        <View>
          <ConfigProvider theme={{ tabsNavBackgroundColor: "transparent" }}>
            <Tabs value={activeTab} onChange={setActiveTab} className="tabs">
              <Tabs.TabPane title="待办理">
                <View className="tabContent">
                  <ScrollView
                    className="card-wrapper"
                    scrollY
                    refresherTriggered={refresherTriggered}
                    refresherEnabled
                    onRefresherRefresh={() => {
                      setRefresherTriggered(true);
                      getData(1);
                    }}
                    refresherBackground="#eff2f6"
                    onScrollToLower={onScrollToLower}
                  >
                    {backlogDataSource.length === 0 && (
                      <Empty>
                        <Empty.Description>无数据</Empty.Description>
                      </Empty>
                    )}
                    {backlogDataSource.map((item) => {
                      return (
                        <View
                          key={item.id}
                          className="card"
                          onClick={onClick.bind(this, item)}
                        >
                          <Flex align="center" className="titleContent">
                            <View className="cardTitle">
                              {item?.subjectName}-{item.name}
                            </View>
                            <Tag style={{whiteSpace:'nowrap'}} color="primary">{item?.caseStatus.name}</Tag>
                          </Flex>
                          <View className="time">
                            申请时间：{item?.createAt}
                          </View>
                        </View>
                      );
                    })}
                  </ScrollView>
                </View>
              </Tabs.TabPane>
              <Tabs.TabPane title="已处理">
                <View className="tabContent">
                  <ScrollView
                    className="card-wrapper"
                    scrollY
                    refresherEnabled
                    refresherBackground="#eff2f6"
                    refresherTriggered={historyRefresherTriggered}
                    onRefresherRefresh={() => {
                      setHistoryRefresherTriggered(true);
                      getHistory(1);
                    }}
                    onScrollToLower={onScrollToLowerInHistory}
                  >
                    {historyDataSource.length === 0 && (
                      <Empty>
                        <Empty.Description>无数据</Empty.Description>
                      </Empty>
                    )}
                    {historyDataSource.map((item) => {
                      return (
                        <View
                          key={item.id}
                          className="card"
                          onClick={onClick.bind(this, item)}
                        >
                          <Flex align="center" className="titleContent">
                            <View className="cardTitle">
                              {item?.subjectName}-{item.name}
                            </View>
                            <Tag style={{whiteSpace:'nowrap'}} color="primary">{item?.caseStatus.name}</Tag>
                          </Flex>
                          <View className="time">
                            申请时间：{item?.createAt}
                          </View>
                        </View>
                      );
                    })}
                  </ScrollView>
                </View>
              </Tabs.TabPane>
            </Tabs>
          </ConfigProvider>
        </View>
      </View>
    </View>
  );
};

export default List;
