import Taro, { useLoad } from "@tarojs/taro";
import { Form, Button, Toast,Popup,Picker } from "@taroify/core";
import { Plus,Cross } from "@taroify/icons";
import useGlobalStore from "../../store/useUserStore";
import { useState, useMemo, useRef } from "react";
import { composeImgUrl, inversionSerializationRule } from "../../utils";
import services from "@/services";
import RenderComponent from "@/components/CustomFormComponents/FormComponentRender";
import api from "@/services/core/api";
import { FormInstance } from "@taroify/core/form";
import "./index.css";
import {
  PageMeta,
  View,
  NavigationBar,
  ScrollView,
  Image,
  Textarea,
  Radio,
} from "@tarojs/components";
import ElectronicSignature, { ElectronicSignatureRef } from "@/components/ElectronicSignature/ElectronicSignature";
import { mySign } from "@/services/commonController";

const Manage = () => {
  const { actionObj, configSource } = useGlobalStore();
  const forwardAndReverseName =
    actionObj?.forwardAndReverseName?.split(",") ?? [];

  const [inputStr, setInputStr] = useState("");
  const [fileList, setFileList] = useState<any[]>([]);

  const [showSelectUser,setShowSelectUser] = useState(false)
  const [tempData,setTempData] = useState<any>({})
  const nodeType = actionObj?.nodeType?.value;
  const formRef = useRef<FormInstance>(null);
  const formConfig = useMemo(() => {
    const matterContentList = configSource?.matterContentList;

    if (matterContentList && actionObj?.examineKeys) {
      let temp = matterContentList
        .sort((pre: any, cur: any) => pre.sort - cur.sort)
        .map((item: any) => {
          return {
            ...item,
            componentOption: item.otherSet ? JSON.parse(item.otherSet) : {},
            key: item.fieldKey,
            validationRules: inversionSerializationRule(item.validationRules),
          };
        });
      /** 过滤需要处理的内容 */
      if (actionObj?.examineKeys !== "all") {
        const keys = actionObj?.examineKeys?.split(",") ?? [];
        temp = temp.filter((item: any) => keys.includes(item.fieldKey));
      } else {
        return [];
      }
      return temp;
    }
    return [];
  }, [configSource, actionObj]);

  const submitReverse = (tagName: string) => {
    const formData = {
      taskId: actionObj?.taskId,
      approveComment: inputStr,
      attachmentList: fileList.join(";"),
      approvePass: "0",
      operationName: tagName,
      approveSignPhoto:signUrl || defaultSign,
      nextSignature:keepUsing?'1':'0'
    };
    if(actionObj.signFlag === '1' &&!signUrl && !defaultSign){
        setSignOpen(true)
        return
    }
    services.login.completeExamine(formData).then((res: any) => {
      if (res.code === 200) {
        Toast.success("处置成功");
        Taro.reLaunch({ url: "/pages/list/index" });
      } else {
        Toast.fail("处置失败");
      }
    });
  };




  const submitExamine = (tagName: string, optional = false) => {
    formRef.current?.validateFields().then((values: any) => {
      const { enableSelectNextNodeUser = false } = actionObj;

      let formData = {
        taskId: actionObj?.taskId,
        approveComment: inputStr,
        attachmentList: fileList.join(";"),
        operationName: tagName,
      } as any;
      if (nodeType === 3 || nodeType === 5) {
        formData = {
          ...formData,
          approvePass: optional ? "2" : "1",
        };
      }

      const params = [] as any[];
  
      for (const key in values) {
        const itemConfig = configSource?.matterContentList?.find(
          (v: any) => v.fieldKey === key
        );

        params.push({
          contentId: key,
          content: values[key],
          applicantRecordId: actionObj.id,
          downFlag:
            itemConfig.componentType === "FileUpload" ||
            itemConfig.componentType === "ImgUpload"
              ? true
              : false,
        });
      }

      const submit = ()=>{
        if(actionObj.signFlag === '1' &&!signUrl && !defaultSign){
            setSignOpen(true)
            return
        }
        services.login.submitForm({
          thingId: actionObj.thingId,
          reportContents: params,
        });
        services.login.completeExamine({...formData,approveSignPhoto:signUrl || defaultSign,nextSignature:keepUsing?'1':'0'}).then((res: any) => {
          if (res.code === 200) {
            Toast.success("处置成功");
            Taro.reLaunch({
              url: "/pages/list/index",
            });
          } else {
            Toast.fail("处置失败");
          }
        });
      }

      if (enableSelectNextNodeUser) {
        services.login.getNextNodeUser(formData).then((res:any) => {
          console.log(res)
          if (!res.data?.nextUsers || res.data?.nextUsers.length === 0) {
            submit()
            return;
          } 
          setTempData({
            ...res.data,
            params: params,
            formData: formData
          })
          setShowSelectUser(true)
        });
        return;
      }
      submit()
    });
  };

  const uploadFile = () => {
    Taro.chooseImage({
      count: 1,
      sizeType: ["original", "compressed"],
      sourceType: ["album", "camera"],
      success: ({ tempFiles }) => {
        Taro.uploadFile({
          url: api.uploadUrl,
          name: "file",
          filePath: tempFiles[0].path,
          success: (res) => {
            const resData = JSON.parse(res.data);
            if (resData.code === 200) {
              setFileList((prev) => [...prev, resData.data]);
            }
          },
        });
      },
    });
  };
/** 删除文件 */
  const delFile = (index:number)=>{
    setFileList(fileList.filter((_,i)=>{
      return i !==index
    }));
  }
  const columns = useMemo(() => {
    return tempData?.nextUsers?.map((v:any)=>{
      return {
        label: v.username,
        value: v.id
      }
    })
  }, [tempData])

  const assignExamine = (userId: string)=>{
    const {params,formData} = tempData
    services.login.submitForm({
      thingId: actionObj.thingId,
      reportContents: params,
    });

    if(actionObj.signFlag === '1' &&!signUrl && !defaultSign){
        setSignOpen(true)
        return
    }
    services.login.completeExamine({...formData,fixUser: userId,approveSignPhoto:signUrl || defaultSign,nextSignature:keepUsing?'1':'0'}).then((res: any) => {
      if (res.code === 200) {
        setShowSelectUser(false)
        Toast.success("处置成功");
        Taro.reLaunch({
          url: "/pages/list/index",
        });
      } else {
        Toast.fail("处置失败");
      }
    });

  }

  const [defaultSign,setDefaultSign] = useState('')
  useLoad(()=>{
    if(actionObj.signFlag === '1'){
      getSign()
    }
  })
  const getSign = async ()=>{
    const res:any = await mySign(Taro.getStorageSync('userId'))
    if(res.code === 200){
      if(res.data !=='无签名'){
        setDefaultSign(res.data)
      }
    }
  }
  const [keepUsing,setKeepUsing] = useState(false)
  const [signOpen,setSignOpen] = useState(false)
  const [signUrl,setSignUrl] = useState('')
  const signRef= useRef<ElectronicSignatureRef>(null)

  return (
    <View>
      <Popup open={showSelectUser} placement="bottom"   rounded style={{ height: "40%" }}>
      <Picker
        title={`选择${tempData.nodeNameAlias}处理人`}
        columns={columns}
        onCancel={()=>{
          setShowSelectUser(false)
        }}
   
        onConfirm={assignExamine}
      >
      </Picker>
      </Popup>
      <PageMeta>
        <NavigationBar
          title="办理"
          backgroundColor="#2E6CFF"
          frontColor={"#fff"}
          style={{ fontSize: 33 }}
        ></NavigationBar>
      </PageMeta>
      <ScrollView scrollY className="manage_container">
        <View style={{ height: 24 }}></View>
        <Toast id="toast" />
        <Form ref={formRef}>
          <View className="formItemGroup">
            {formConfig.map((formItem: any) => {
              return (
                <Form.Item
                  name={formItem.key}
                  rules={[
                    {
                      required: formItem.componentOption?.require,
                      message: `请填写${formItem?.label}`,
                    },
                    ...formItem.validationRules,
                  ]}
                >
                  {formItem.componentType === "Branch" ? undefined : (
                    <Form.Label>{formItem.label}</Form.Label>
                  )}

                  <Form.Control>
                    {(controller) => (
                      <RenderComponent
                        config={formItem.componentOption}
                        componentType={formItem.componentType}
                        metaData={formItem}
                        {...controller}
                      ></RenderComponent>
                    )}
                  </Form.Control>
                </Form.Item>
              );
            })}
          </View>

          <View className="examineWrapper">
            <Textarea
              className="examineWrapper_area"
              placeholder="请输入审批意见"
              onInput={(e) => {
                setInputStr(e.detail.value);
              }}
            />
            <View className="attachmentList">
              {fileList?.map((item,index) => {
                return (
                  <View className="attachment_upload">
                    <View className="attachment_del" onTap={()=>delFile(index)}>
                    <Cross />
                    </View>
                    <Image
                      src={composeImgUrl(item)}
                      onClick={() => {
                        Taro.previewImage({
                          current: composeImgUrl(item),
                          urls: [composeImgUrl(item)],
                        });
                      }}
                      style={{ width: "100%", height: "100%" }}
                    />
                  </View>
                );
              })}
              <View className="attachment_upload" onClick={uploadFile}>
                <Plus />
              </View>
            </View>
          </View>
          <View className="actionWrapper_container">
            {nodeType === 2 && (
              <Button
                shape="round"
                block
                color="primary"
                onClick={() => submitExamine("通过")}
              >
                通过
              </Button>
            )}
            {nodeType === 3 && (
              <>
                <Button
                  shape="round"
                  block
                  color="danger"
                  onClick={() => submitReverse(forwardAndReverseName[1])}
                >
                  {forwardAndReverseName[1]}
                </Button>
                <Button
                  shape="round"
                  block
                  color="primary"
                  onClick={() => submitExamine(forwardAndReverseName[0])}
                >
                  {forwardAndReverseName[0]}
                </Button>
              </>
            )}
            {nodeType === 5 && (
              <>
                <Button
                  shape="round"
                  block
                  color="danger"
                  onClick={() => submitReverse(forwardAndReverseName[2])}
                >
                  {forwardAndReverseName[2]}
                </Button>
                <Button
                  shape="round"
                  block
                  color="info"
                  onClick={() => submitExamine(forwardAndReverseName[0], true)}
                >
                  {forwardAndReverseName[0]}
                </Button>
                <Button
                  shape="round"
                  block
                  color="primary"
                  onClick={() => submitExamine(forwardAndReverseName[1])}
                >
                  {forwardAndReverseName[1]}
                </Button>
              </>
            )}
            {/* <Button shape='round' color='primary' onClick={()=>setSignOpen(true)}>电子签名</Button> */}
            {signUrl || defaultSign?<Image className="signImg" mode='widthFix' src={api.assetsUrl + (signUrl || defaultSign)} onClick={()=>setSignOpen(true)}/>:<Button shape='round' color='primary' onClick={()=>setSignOpen(true)}>电子签名</Button>} 
          </View>
          <View style={{ height: 50 }}></View>
        </Form>
      </ScrollView>
       <Popup
            open={signOpen}
            rounded
            placement='bottom'
            className="signPopup"
        >
        <View className="signTitle">
            电子签名
            <Button style={{flex:'initial',height:'100%'}} color='primary' onClick={()=>setSignOpen(false)}>取消</Button>
        </View>
        <View className="signBox">
            <View className="canvasBox">
                <ElectronicSignature ref={signRef} width="100%" height="100%" bgColor="#F4F4F6" defaultSign={defaultSign} />
            </View>
            <View className="ridioBox">
                <Radio checked={keepUsing} onClick={()=>setKeepUsing(!keepUsing)}>下次同意继续使用此签名</Radio>
                <View className="clearSign" onClick={()=>{
                   setSignUrl('')
                   signRef.current?.resetDraw() 
                }}>清除签名</View>
            </View>
        </View>
        <Button className="overBtn" onClick={async()=>{
            const filePath =await signRef.current?.subCanvas()
            setSignUrl(filePath)
            setSignOpen(false)
        }}>完成</Button>
      </Popup>
    </View>
  );
};

export default Manage;
